# -*- coding: utf-8 -*-

__doc__ = """Adds flavor of interactive filtering to the traditional pipe concept of shell.
Try::

    $ A | percol | B

and you can display the output of command A and filter it interactively and then pass it to command B.
The interface of percol is highly inspired by anything.el for Emacs.

Full details at https://github.com/mooz/percol"""

__version__ = "0.2.1"

__logo__ = """                                    __
        ____  ___  ______________  / /
       / __ \/ _ \/ ___/ ___/ __ \/ /
      / /_/ /  __/ /  / /__/ /_/ / /
     / .___/\___/_/   \___/\____/_/
    /_/"""
